@@ -3,6 +3,8 @@
  * import vdr channels.conf files
  */
 
+#include <features.h>
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -19,6 +21,11 @@
 
 #define BUFFSIZE	4096
 
+// If large file support is not included, then make the value do nothing
+#ifndef O_LARGEFILE
+#define O_LARGEFILE	0
+#endif
+
 /* ----------------------------------------------------------------------- */
 int write_stream(struct dvb_state *h, char *filename, int sec)
 {
@@ -35,7 +42,7 @@ int write_stream(struct dvb_state *h, char *filename, int sec)
 		exit(1);
     }
     
-    file = open(filename, O_WRONLY | O_TRUNC | O_CREAT, 0666);
+    file = open(filename, O_WRONLY | O_TRUNC | O_CREAT | O_LARGEFILE, 0666);
     if (-1 == file) {
 		fprintf(stderr,"open %s: %s\n",filename,strerror(errno));
 		exit(1);
